import {Component, Input, OnInit} from '@angular/core';
import {CapabilityVersion} from "../../../../model/capability-version";
import {Capability} from "../../../../model/capability";
import {CapabilityRequestService} from "../../../../services/capability-request.service";
import {CapabilityRequest} from "../../../../model/capability-request";
import {CapabilitiesService} from "../../../../services/capabilities.service";

@Component({
  selector: 'app-qa-controls',
  templateUrl: './qa-controls.component.html',
  styleUrls: ['./qa-controls.component.scss']
})
export class QaControlsComponent implements OnInit {

  @Input() public capability: Capability;
  @Input() public capabilityRequest: CapabilityRequest;
  @Input() public currentVersion: CapabilityVersion;
  public defaultCC: string;

  constructor(
    private capabilitiesService: CapabilitiesService,
    private capabilityRequestService: CapabilityRequestService,
  ) {}

  ngOnInit(): void {
    this.loadDefaultCC();
  }


  /**
   * Stage 1 QAPass this request (single or double QA)
   * - In single case: runs QAPass state machine action
   *  - In double case: sends version to Stage 2 review
   *
   * @param version the version of the request on which to pass QA
   */
  public passQa(version: CapabilityVersion) {
    const addRestCallObservable = {
      next: (response) => {
        console.log(">>> QA Pass response:'" + response + "'");
      },
      error: (error) =>
        console.error(
          "Error when trying to pass QA on request " +
          version.current_execution.current_workflow_request_id +
          ": " +
          error,
        ),
    };

    return this.capabilityRequestService
      .sendQaPassRequest(
        parseInt(version.current_execution.current_workflow_request_id),
        version.version_number,
      )
      .subscribe(addRestCallObservable);
  }

  /**
   * Stage 1 QAFail this request (runs QAFail workflow for specified version, all QA cases)
   *
   * @param version the version of the request on which to pass QA
   */
  public failQa(version: CapabilityVersion) {
    const addRestCallObservable = {
      next: (response) => {
        console.log(">>> QA Fail response:'" + response + "'");
      },
      error: (error) =>
        console.error(
          "Error when trying to fail QA on workflow request " +
          version.current_execution.current_workflow_request_id +
          ": " +
          error,
        ),
    };

    return this.capabilityRequestService
      .sendQaFailRequest(
        parseInt(version.current_execution.current_workflow_request_id),
        version.version_number,
      )
      .subscribe(addRestCallObservable);
  }

  /**
   * Abandon (i.e. fail all outstanding versions) QA for this Request
   * - Same functionality for all QA cases, Stage 1 level only on Double QA
   *
   * @param version Version of the Request from which the request was sent.
   */
  public abandonQa(version: CapabilityVersion) {
    const addRestCallObservable = {
      next: (response) => {
        console.log(">>> QA Abandon response:'" + response + "'");
      },
      error: (error) =>
        console.error(
          "Error when trying to abandon QA on workflow request " +
          version.current_execution.current_workflow_request_id +
          ": " +
          error,
        ),
    };

    return this.capabilityRequestService
      .sendQaAbandonRequest(
        parseInt(version.current_execution.current_workflow_request_id),
        version.version_number,
      ).subscribe(addRestCallObservable);
  }


  /**
   * Stage 2 confirmation of QA results
   * - Double QA Only: Runs QAPass state machine action
   *
   * @param version
   */
  public confirmQa(version: CapabilityVersion){
    const addRestCallObservable = {
      next: (response) => {
        console.log(">>> Stage 2 Pass response:'" + response + "'");
      },
      error: (error) =>
        console.error(
          "Error when trying to pass Stage 2 Review on request " +
          version.current_execution.current_workflow_request_id +
          ": " +
          error,
        ),
    };

    return this.capabilityRequestService
      .sendStage2QaPassRequest(
        parseInt(version.current_execution.current_workflow_request_id),
        version.version_number,
      )
      .subscribe(addRestCallObservable);
  }

  /**
   * Stage 2 Failure, return results to Stage 1
   * - Double QA Only: Returns request to Awaiting QA state and notifies assigned Stage 1 reviewer
   *
   * @param version
   */
  public revisitQa(version: CapabilityVersion){
    const addRestCallObservable = {
      next: (response) => {
        console.log(">>> QA Revisit response:'" + response + "'");
      },
      error: (error) =>
        console.error(
          "Error when trying to reset to Stage 1 for QA review on request " +
          version.current_execution.current_workflow_request_id +
          ": " +
          error,
        ),
    };

    return this.capabilityRequestService
      .sendQaRevisitRequest(
        parseInt(version.current_execution.current_workflow_request_id),
        version.version_number,
      )
      .subscribe(addRestCallObservable);
  }

  public loadDefaultCC() {
    const getAnalystEmailObserver = {
      next: (response) => {
        if (response.resp) {
          this.defaultCC = "schedsoc@nrao.edu," + response.resp;
        }
      },
      error: (error) => {
        console.error("Failed to load analyst email:", error);
        this.defaultCC = "schedsoc@nrao.edu";
      }
    };

    return this.capabilitiesService.getAnalystEmail().subscribe(getAnalystEmailObserver);
  }

  public getEmailParams() {
    return {
      "destination_email": this.currentVersion.parameters.user_email,
      "version": this.currentVersion,
      "workflow_metadata": this.currentVersion.workflow_metadata
    };
  }
}
